"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const querystring_1 = __importDefault(require("querystring"));
const electron_1 = require("electron");
const electron_context_menu_1 = __importDefault(require("electron-context-menu"));
const menu_1 = require("./menu");
const about_window_1 = __importDefault(require("about-window"));
const store_1 = require("./store");
const keyBindings_1 = require("./keyBindings");
const utils_1 = require("./utils");
const config = require(path.resolve(__dirname, 'config.json'));
let appUrl = config.TANA_ELECTRON_URL;
if (store_1.preferencesStore.has('webClientEnv')) {
    const storedWebClientEnv = store_1.preferencesStore.get('webClientEnv');
    if (storedWebClientEnv === 'main') {
        appUrl = 'https://be.tana.inc';
    }
}
electron_1.app.setName('Tana');
const keyBindingManager = new keyBindings_1.KeyBindingManager(electron_1.globalShortcut, (command) => {
    if (command === 'voiceCapture') {
        if (!isCapturing) {
            isCapturing = true;
            mainWindow?.webContents.send('voice-capture', { action: 'start' });
        }
        else {
            isCapturing = false;
            mainWindow?.webContents.send('voice-capture', { action: 'stop' });
        }
    }
});
// https://www.electronforge.io/config/makers/squirrel.windows
if (require('electron-squirrel-startup')) {
    electron_1.app.quit();
}
function isSpellCheckerEnabled() {
    return store_1.preferencesStore.get('isSpellCheckerEnabled');
}
const server = 'https://desktop-update.tana.inc';
const updateServerUrl = `${server}/update/${process.platform}/${electron_1.app.getVersion()}`;
const isDevMode = process.env.NODE_ENV === 'development';
let updatePollInterval;
electron_1.autoUpdater.setFeedURL({ url: updateServerUrl });
electron_1.autoUpdater.on('update-available', () => {
    console.log("There's an update available");
    // stop polling
    clearInterval(updatePollInterval);
});
electron_1.autoUpdater.on('error', (err) => {
    // we need this so that the autoUpdater doesn't throw an error (dialog) when there's no internet connection
});
electron_1.autoUpdater.on('update-downloaded', (event, releaseNotes, releaseName) => {
    const dialogOpts = {
        type: 'info',
        buttons: ['Restart', 'Later'],
        title: 'Application Update',
        message: process.platform === 'win32' ? releaseNotes : releaseName,
        detail: 'A new version has been downloaded. Restart the application to apply the updates.',
    };
    if (mainWindow) {
        electron_1.dialog.showMessageBox(mainWindow, dialogOpts).then((returnValue) => {
            if (returnValue.response === 0) {
                electron_1.autoUpdater.quitAndInstall();
            }
        });
    }
});
const env = (0, utils_1.getEnvFromUrl)(appUrl);
if (!appUrl) {
    console.error(`${appUrl} is not supported`);
    throw new Error(`${appUrl} is not supported`);
}
console.log('Starting against', appUrl);
const openDevtoolsByDefault = false;
(0, electron_context_menu_1.default)({
    showSaveImageAs: true,
    prepend: (defaultActions, parameters, browserWindow) => [
        {
            label: 'Back',
            visible: mainWindow?.webContents.canGoBack(),
            click: () => {
                mainWindow?.webContents.goBack();
            },
        },
        {
            label: 'Forward',
            visible: mainWindow?.webContents.canGoForward(),
            click: () => {
                mainWindow?.webContents.goForward();
            },
        },
        {
            label: 'Save as…',
            visible: parameters.mediaType === 'plugin' && parameters.mediaFlags.canSave,
            click: () => {
                mainWindow?.webContents?.downloadURL(parameters.frameURL);
            },
        },
    ],
});
let mainWindow;
let searchWindow;
let keyboardBindingsWindow;
let voiceRecorderWindow;
function showSearchWindow() {
    if (!mainWindow) {
        return;
    }
    if (!searchWindow) {
        getOrCreateSearchWindow();
        if (!searchWindow) {
            return;
        }
    }
    searchWindow.show();
    // relocate window
    searchWindow.setContentBounds({
        x: (mainWindow.getContentBounds().x || 0) + mainWindow.getContentBounds().width - 380,
        y: (mainWindow.getContentBounds().y || 0) + 10,
        width: 380,
        height: 70,
    });
    searchWindow?.webContents.send('find-in-page', {
        action: 'focus',
    });
}
function getOrCreateSearchWindow() {
    if (!searchWindow) {
        searchWindow = new electron_1.BrowserWindow({
            title: 'Tana',
            transparent: true,
            frame: false,
            resizable: false,
            webPreferences: {
                preload: path.resolve(__dirname, 'preload.js'),
            },
        });
        if (openDevtoolsByDefault) {
            searchWindow.webContents.openDevTools({ mode: 'undocked' });
        }
        searchWindow.on('closed', () => {
            searchWindow = undefined;
        });
        (0, utils_1.loadIntoWindow)(searchWindow, 'findInPage', isDevMode);
        searchWindow.setAlwaysOnTop(true);
        searchWindow.hide();
    }
}
function getOrCreateCustomizeKeyboarBindingsWindow() {
    if (!keyboardBindingsWindow) {
        keyboardBindingsWindow = new electron_1.BrowserWindow({
            title: 'Customize Keyboard Bindings',
            resizable: false,
            width: 650,
            height: 500,
            webPreferences: {
                preload: path.resolve(__dirname, 'preload.js'),
            },
        });
        if (openDevtoolsByDefault) {
            keyboardBindingsWindow.webContents.openDevTools();
        }
        keyboardBindingsWindow.on('closed', () => {
            keyboardBindingsWindow = undefined;
        });
        (0, utils_1.loadIntoWindow)(keyboardBindingsWindow, 'keyboardBindings', isDevMode);
        keyboardBindingsWindow.setAlwaysOnTop(true);
        keyboardBindingsWindow.hide();
    }
}
function showRecorderWindow() {
    if (!mainWindow) {
        return;
    }
    if (!voiceRecorderWindow) {
        getOrCreateVoiceRecorderWindow();
        if (!voiceRecorderWindow) {
            return;
        }
    }
    if (!voiceRecorderWindow.isVisible()) {
        voiceRecorderWindow.show();
    }
    // relocate window
    voiceRecorderWindow.setContentBounds({
        x: Math.round((mainWindow.getContentBounds().x || 0) + mainWindow.getContentBounds().width / 2 - 160 / 2),
        y: (mainWindow.getContentBounds().y || 0) + mainWindow.getContentBounds().height - 34 - 16,
        width: 160,
        height: 34,
    });
}
function getOrCreateVoiceRecorderWindow() {
    if (!voiceRecorderWindow) {
        voiceRecorderWindow = new electron_1.BrowserWindow({
            title: 'Capture Voice',
            transparent: true,
            frame: false,
            resizable: false,
            skipTaskbar: true,
            alwaysOnTop: true,
            webPreferences: {
                preload: path.resolve(__dirname, 'preload.js'),
            },
        });
        voiceRecorderWindow.on('closed', () => {
            voiceRecorderWindow = undefined;
        });
        (0, utils_1.loadIntoWindow)(voiceRecorderWindow, 'voiceRecorder', isDevMode);
        voiceRecorderWindow.setAlwaysOnTop(true);
    }
}
if (process.defaultApp) {
    if (process.argv.length >= 2) {
        electron_1.app.setAsDefaultProtocolClient('tana', process.execPath, [path.resolve(process.argv[1])]);
    }
}
else {
    electron_1.app.setAsDefaultProtocolClient('tana');
}
const isMac = process.platform === 'darwin';
function buildAndSetApplicationMenu() {
    const menu = electron_1.Menu.buildFromTemplate((0, menu_1.getMenuTemplate)({
        isMac,
        isSpellCheckerEnabled: isSpellCheckerEnabled(),
        onAction: (action) => {
            if (action === 'search') {
                showSearchWindow();
            }
            else if (action === 'back') {
                mainWindow?.webContents.goBack();
            }
            else if (action === 'forward') {
                mainWindow?.webContents.goForward();
            }
            else if (action === 'about') {
                (0, about_window_1.default)({
                    homepage: 'https://tana.inc',
                    use_version_info: true,
                    icon_path: path.resolve(__dirname, '../assets/icons/icon.png'),
                    product_name: 'Tana',
                    // show env unless prod
                    description: env !== 'stable' ? (0, utils_1.getEnvFromUrl)(appUrl) : 'The all-in-one workspace',
                });
            }
            else if (action === 'toggleSpellCheck') {
                const currentState = isSpellCheckerEnabled();
                store_1.preferencesStore.set('isSpellCheckerEnabled', !currentState);
                mainWindow?.webContents.session.setSpellCheckerEnabled(!currentState);
                buildAndSetApplicationMenu();
            }
            else if (action === 'keyBindings') {
                getOrCreateCustomizeKeyboarBindingsWindow();
                keyboardBindingsWindow?.show();
            }
            else {
                console.warn('Unknown action', action);
            }
        },
    }));
    electron_1.Menu.setApplicationMenu(menu);
}
buildAndSetApplicationMenu();
const gotTheLock = electron_1.app.requestSingleInstanceLock();
if (!gotTheLock) {
    electron_1.app.quit();
}
else {
    electron_1.app.on('second-instance', (event, commandLine, workingDirectory) => {
        if (!mainWindow) {
            return;
        }
        if (mainWindow.isMinimized()) {
            mainWindow.restore();
        }
        mainWindow.focus();
        const url = commandLine.pop()?.slice(0, -1);
        if (url?.startsWith('tana://auth')) {
            onAuth(url);
        }
        console.log('Welcome Back', `You arrived from: ${url}`);
    });
}
const createWindow = async () => {
    mainWindow = new electron_1.BrowserWindow({
        title: 'Tana',
        webPreferences: {
            spellcheck: isSpellCheckerEnabled(),
            preload: path.resolve(__dirname, 'preload.js'),
        },
        autoHideMenuBar: true,
    });
    const mainWindowBounds = store_1.preferencesStore.get('mainWindowBounds');
    if (mainWindowBounds) {
        mainWindow.setBounds(mainWindowBounds);
    }
    else {
        mainWindow.maximize();
    }
    if (openDevtoolsByDefault) {
        mainWindow.webContents.openDevTools();
    }
    mainWindow.webContents.setVisualZoomLevelLimits(1, 3);
    mainWindow.webContents.setWindowOpenHandler((details) => {
        if (details.url === 'about:blank') {
            return { action: 'allow' };
        }
        if (details.url.startsWith('https://accounts.google.com')) {
            return { action: 'allow' };
        }
        electron_1.shell.openExternal(details.url);
        return { action: 'deny' };
    });
    await mainWindow.loadURL(appUrl);
    mainWindow.on('close', () => {
        if (mainWindow) {
            store_1.preferencesStore.set('mainWindowBounds', mainWindow.getBounds());
        }
    });
    // quit application when main window is closed
    mainWindow.on('closed', () => {
        electron_1.app.quit();
    });
    // we found content on the page, let the search view know so it can update the UI
    mainWindow.webContents.on('found-in-page', (event, result) => {
        searchWindow?.webContents.send('find-in-page', {
            numMatches: result.matches,
            activeMatch: result.activeMatchOrdinal,
        });
    });
    mainWindow?.webContents.on('before-input-event', (event, input) => {
        const isMetaOrCtrl = isMac ? input.meta : input.control;
        const isShift = input.shift;
        const inputKey = input.key.toLowerCase();
        if (isMetaOrCtrl && inputKey === 'z') {
            event.preventDefault();
            if (!isShift) {
                mainWindow?.webContents.send('undo');
            }
            else {
                mainWindow?.webContents.send('redo');
            }
        }
    });
    electron_1.ipcMain.on('auth_request', () => {
        electron_1.shell.openExternal(`${appUrl}/electron`);
    });
    electron_1.ipcMain.on('voice-data', (event, args) => {
        if (!voiceRecorderWindow) {
            return;
        }
        voiceRecorderWindow?.webContents.send('voice-data', args);
    });
    electron_1.ipcMain.on('voice-capture', (event, args) => {
        if (args.action === 'start') {
            if (!voiceRecorderWindow) {
                showRecorderWindow();
            }
        }
        else if (args.action === 'stop') {
            // prevent infinite messages loop
            if (event.sender === voiceRecorderWindow?.webContents) {
                mainWindow?.webContents.send('voice-capture', { action: 'stop' });
            }
            voiceRecorderWindow?.close();
            voiceRecorderWindow = undefined;
        }
    });
    electron_1.ipcMain.on('custom-keybindings', (event, args) => {
        if (args.action === 'ready') {
            keyboardBindingsWindow?.webContents.send('custom-keybindings', {
                action: 'init',
                bindings: keyBindingManager.getKeyBindings(),
            });
        }
        else if (args.action === 'update') {
            keyboardBindingsWindow?.close();
            keyboardBindingsWindow = undefined;
            keyBindingManager.updateKeyBindings(args.bindings);
        }
    });
    // find in page triggered by input in the search box
    electron_1.ipcMain.on('find-in-page', (event, args) => {
        if (args.action === 'close') {
            mainWindow?.webContents.stopFindInPage('clearSelection');
            // close search window
            searchWindow?.hide();
        }
        // we need a search term
        if (args.term) {
            if (args.action === 'search') {
                mainWindow?.webContents.findInPage(args.term, { findNext: true });
            }
            else if (args.action === 'next') {
                mainWindow?.webContents.findInPage(args.term, { forward: true });
            }
            else if (args.action === 'previous') {
                mainWindow?.webContents.findInPage(args.term, { forward: false });
            }
        }
    });
    electron_1.ipcMain.on('theme', (event, args) => {
        if (args.themeName === 'dark') {
            electron_1.nativeTheme.themeSource = 'dark';
        }
        else if (args.themeName === 'light') {
            electron_1.nativeTheme.themeSource = 'light';
        }
        else {
            electron_1.nativeTheme.themeSource = 'system';
        }
    });
    electron_1.ipcMain.on('set-env', (event, args) => {
        if (['main', 'stable'].includes(args.env)) {
            store_1.preferencesStore.set('webClientEnv', args.env);
        }
    });
};
let isCapturing = false;
electron_1.app
    .whenReady()
    .then(() => {
    // start polling after five
    setTimeout(() => {
        //poll for updates every 15 minutes
        updatePollInterval = setInterval(() => {
            console.log('Checking for updates');
            try {
                electron_1.autoUpdater.checkForUpdates();
            }
            catch (e) {
                // eslint-disable-next-line local-rules/disallow-console-error-no-message
                console.error(e);
            }
        }, 15 * 60 * 1000);
    }, 5 * 60 * 1000);
    keyBindingManager.bindAll();
})
    .then(() => {
    createWindow().then(() => {
        getOrCreateSearchWindow();
        electron_1.app.on('activate', () => {
            if (electron_1.BrowserWindow.getAllWindows().length === 0) {
                createWindow();
            }
        });
    });
});
electron_1.app.on('window-all-closed', () => {
    if (process.platform !== 'darwin') {
        electron_1.app.quit();
    }
});
electron_1.app.on('open-url', (event, url) => {
    console.log('Welcome Back', `You arrived from: ${url}`);
    if (url.startsWith('tana://auth')) {
        onAuth(url);
    }
    if (url.startsWith('tana://node')) {
        onNodeOpen(url);
    }
});
// tana://node/<nodeId>
const onNodeOpen = (url) => {
    const nodeId = url.replace('tana://node/', '');
    mainWindow?.webContents.send('open-node', { nodeId });
};
const onAuth = (returnUrl) => {
    // remove extra slash added on Windows and Linux
    if (returnUrl.startsWith('tana://auth/?')) {
        returnUrl = returnUrl.replace('tana://auth/?', 'tana://auth?');
    }
    const parts = returnUrl.split('auth?');
    if (parts.length < 2) {
        console.log("Can't parse auth url", returnUrl);
        return;
    }
    const { token, providerId } = querystring_1.default.parse(parts[1]);
    mainWindow?.webContents.send('auth', { token, providerId });
};
//# sourceMappingURL=main.js.map